define("visual/Grid",
	["visual/TextLine", "d3"], 
	function(TextLine){
		/**
		 * @constructor
		 */
		var Grid = function(/*string|dom*/node, opt){
			this.node = d3.select(node);
			this.rowNum = opt.rowNum;
			this.colNum = opt.colNum;
			this.nodeSize = opt.nodeSize; // in unit of pixel
			this.textLines = {};
			this.container = this.node.append("svg");
		};

		Grid.prototype.init = function(){
			var i, j,x=0, y=0;
			for(i=0; i<this.rowNum; i++){
				y = this.nodeSize * i;
				for(j=0; j<this.colNum; j++){
					x = this.nodeSize * j;
					this.container.append("rect")
								  .attr("x", x)
								  .attr("y", y)
								  .attr("width", this.nodeSize)
								  .attr("height", this.nodeSize)
								  .attr("fill", "#FFF")
								  .attr("stroke", "#000")
								  .attr("stroke-opacity", 0.2);
				}
			}
		};

		Grid.prototype.clear = function(){
			this.textLines = [];
			this.init();
		};

		Grid.prototype.drawTextLine = function(/*string*/name, /*string*/text, x, y){
			if(this.textLines[name]){
				this.textLines[name].text =text;
				this.textLines[name].x =x;
				this.textLines[name].x =y;
			}else{
				this.textLines[name] = new TextLine({
					name: name,
					text: text,
					x: x,
					y: y,
					container: this.container,
					nodeSize: this.nodeSize
				});
			}
			
			this.textLines[name].draw();

			return this.textLines[name];
		};

		return Grid;
	}
);