define("visual/TextLine",
	[], 
	function(){
	/**
	 * TextLine is a line of nodes it can be generated and moved as a whole
	 * @constructor
	 */
	var TextLine = function(opt){
		this.container = opt.container; //svg element is the container
		this.text = opt.text;
		this.left = opt.x;
		this.top = opt.y;
		this.name = opt.name;
		this.nodeSize = opt.nodeSize;
		this.textRects = [];
	};

	var proto = TextLine.prototype;

	proto.draw = function(){
		this.clear();
		var y = this.top*this.nodeSize - 6,
			chars = this.text.split(""),
			i, c, x, node, 
			length = chars.length;
		for(i=0; i<length; i++){
			x = (this.left + i) * this.nodeSize + 6;
			c = chars[i];
			node = this.container.append("text")
								 .text(c)
								 .attr("x", x)
								 .attr("y", y)
								 .attr("font-family", "verdana")
								 .attr("font-size", this.nodeSize - 4)
								 .transition()
								 .attr("fill", "#999")
								 .node();
			this.textRects[i] = node;
		}
	};

	proto.moveRight = function(steps){
		var i, x,
			length = this.textRects.length;
		this.left = this.left + steps;

		for(i=0;i<length;i++){
			x = (this.left + i) * this.nodeSize + 6;
			d3.select(this.textRects[i]).attr("x", x);
		}
	};

	proto.moveLeft = function(steps){
		this.moveRight(-steps);
	};

	proto.clear = function(){
		var i, length = this.textRects.length;
		for(i=0; i<length; i++){
			d3.select(this.textRects[i]).remove();
		}
	};

	proto.markMatched = function(index, end){
		var i;
		if(typeof(end) === "undefined"){
			var end = index + 1;
		}
		for(i=index; i<end; i++){
			this._setColor(i, "green");
		}
	};

	proto.markFailed = function(index, end){
		var i;
		if(typeof(end) === "undefined"){
			var end = index + 1;
		}
		for(i=index; i<end; i++){
			this._setColor(i, "red");
		}
	};

	proto._setColor = function(index, color){
		 d3.select(this.textRects[index]).transition()
		 								 .attr("fill", color);
	};

	return TextLine;
});